@@ -69,11 +69,13 @@ signupSupport = {{signup_support}}
 allowChangeSigninMode = {{allow_change_signin_mode}}
 allowAnonymousChangePassword = {{allow_anonymous_change_password}}
 allowProjectResourceMonitor = {{allow_project_resource_monitor}}
+allowManualImageNameForSession = {{allow_manual_image_name_for_session}}
 autoLogout = {{auto_logout}}
 
 [resources]
 openPortToPublic = {{open_port_to_public}}
 maxCPUCoresPerContainer = {{max_cpu_cores_per_container}}
+maxMemoryPerContainer = {{max_memory_per_container}}
 maxCUDADevicesPerContainer = {{max_cuda_devices_per_container}}
 maxCUDASharesPerContainer = {{max_cuda_shares_per_container}}
 maxShmPerContainer = {{max_shm_per_container}}
@@ -147,6 +149,7 @@ async def console_handler(request: web.Request) -> web.StreamResponse:
         if 'resources' in config:
             open_port_to_public = 'true' if config['resources'].get('open_port_to_public') else 'false'
             max_cpu_cores_per_container = config['resources'].get('max_cpu_cores_per_container', 64)
+            max_memory_per_container = config['resources'].get('max_memory_per_container', 64)
             max_cuda_devices_per_container = config['resources'].get(
                     'max_cuda_devices_per_container', 16)
             max_cuda_shares_per_container = config['resources'].get(
@@ -156,6 +159,7 @@ async def console_handler(request: web.Request) -> web.StreamResponse:
         else:
             open_port_to_public = 'false'
             max_cpu_cores_per_container = 64
+            max_memory_per_container = 64
             max_cuda_devices_per_container = 16
             max_cuda_shares_per_container = 16
             max_shm_per_container = 2
@@ -178,10 +182,13 @@ async def console_handler(request: web.Request) -> web.StreamResponse:
                 'true' if config['service'].get('allow_anonymous_change_password') else 'false',
             'allow_project_resource_monitor':
                 'true' if config['service']['allow_project_resource_monitor'] else 'false',
+            'allow_manual_image_name_for_session':
+                'true' if config['service'].get('allow_manual_image_name_for_session') else 'false',
             'auto_logout':
-                'true' if config['service'].get('auto_logout') else 'false',
+                'true' if config['session'].get('auto_logout') else 'false',
             'open_port_to_public': open_port_to_public,
             'max_cpu_cores_per_container': max_cpu_cores_per_container,
+            'max_memory_per_container': max_memory_per_container,
             'max_cuda_devices_per_container': max_cuda_devices_per_container,
             'max_cuda_shares_per_container': max_cuda_shares_per_container,
             'max_shm_per_container': max_shm_per_container,
@@ -381,6 +388,93 @@ async def logout_handler(request: web.Request) -> web.Response:
     return web.Response(status=201)
 
 
+async def webserver_healthcheck(request: web.Request) -> web.Response:
+    result = {
+        'version': __version__,
+        'details': 'Success'
+    }
+    return web.json_response(result)
+
+
+async def token_login_handler(request: web.Request) -> web.Response:
+    config = request.app['config']
+
+    # Check browser session exists.
+    session = await get_session(request)
+    if session.get('authenticated', False):
+        return web.HTTPBadRequest(text=json.dumps({
+            'type': 'https://api.backend.ai/probs/generic-bad-request',
+            'title': 'You have already logged in.',
+        }), content_type='application/problem+json')
+
+    # Check if auth token is delivered through cookie.
+    auth_token_name = config['api'].get('auth_token_name')
+    if not auth_token_name:
+        return web.HTTPBadRequest(text=json.dumps({
+            'type': 'https://api.backend.ai/probs/invalid-api-params',
+            'title': 'Auth token name is not defined',
+        }), content_type='application/problem+json')
+    auth_token = request.cookies.get(auth_token_name)
+    if not auth_token:
+        return web.HTTPBadRequest(text=json.dumps({
+            'type': 'https://api.backend.ai/probs/invalid-api-params',
+            'title': 'You must provide cookie-based authentication token',
+        }), content_type='application/problem+json')
+
+    # Login with the token.
+    # We do not pose consecutive login failure for this handler since
+    # user may frequently click edu-api launcher button.
+    result: MutableMapping[str, Any] = {
+        'authenticated': False,
+        'data': None,
+    }
+    try:
+        anon_api_config = APIConfig(
+            domain=config['api']['domain'],
+            endpoint=config['api']['endpoint'],
+            access_key='', secret_key='',  # anonymous session
+            user_agent=user_agent,
+            skip_sslcert_validation=not config['api'].get('ssl-verify', True),
+        )
+        assert anon_api_config.is_anonymous
+        async with APISession(config=anon_api_config) as api_session:
+            # Instead of email and password, cookie token will be used for auth.
+            api_session.aiohttp_session.cookie_jar.update_cookies(request.cookies)
+            token = await api_session.User.authorize('fake-email', 'fake-pwd')
+            stored_token = {
+                'type': 'keypair',
+                'access_key': token.content['access_key'],
+                'secret_key': token.content['secret_key'],
+                'role': token.content['role'],
+                'status': token.content.get('status'),
+            }
+            public_return = {
+                'access_key': token.content['access_key'],
+                'role': token.content['role'],
+                'status': token.content.get('status'),
+            }
+            session['authenticated'] = True
+            session['token'] = stored_token  # store full token
+            result['authenticated'] = True
+            result['data'] = public_return  # store public info from token
+    except BackendClientError as e:
+        return web.HTTPBadGateway(text=json.dumps({
+            'type': 'https://api.backend.ai/probs/bad-gateway',
+            'title': "The proxy target server is inaccessible.",
+            'details': str(e),
+        }), content_type='application/problem+json')
+    except BackendAPIError as e:
+        log.info('Authorization failed for token {}: {}', auth_token, e)
+        result['authenticated'] = False
+        result['data'] = {
+            'type': e.data.get('type'),
+            'title': e.data.get('title'),
+            'details': e.data.get('msg'),
+        }
+        session['authenticated'] = False
+    return web.json_response(result)
+
+
 async def server_shutdown(app):
     pass
 
@@ -425,8 +519,10 @@ async def server_main(loop, pidx, args):
     app.router.add_route('PATCH', '/func/{path:folders/_/tus/upload/.*$}', anon_web_plugin_handler)
     app.router.add_route('OPTIONS', '/func/{path:folders/_/tus/upload/.*$}', anon_web_plugin_handler)
     cors.add(app.router.add_route('POST', '/server/login', login_handler))
+    cors.add(app.router.add_route('POST', '/server/token-login', token_login_handler))
     cors.add(app.router.add_route('POST', '/server/login-check', login_check_handler))
     cors.add(app.router.add_route('POST', '/server/logout', logout_handler))
+    cors.add(app.router.add_route('GET', '/func/ping', webserver_healthcheck))
     cors.add(app.router.add_route('GET', '/func/{path:hanati/user}', anon_web_plugin_handler))
     cors.add(app.router.add_route('GET', '/func/{path:cloud/.*$}', anon_web_plugin_handler))
     cors.add(app.router.add_route('POST', '/func/{path:cloud/.*$}', anon_web_plugin_handler))
